/*
Part of the algoviz@vt collection of algorithm visualizations.
Copyright (C) 2007  Virginia Tech Department of Computer Science

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/
import javax.swing.*;
import javax.swing.BorderFactory.*;
import javax.swing.text.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.event.ActionEvent;
import java.io.*;
import java.lang.Integer;
import javax.swing.JComponent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.*;
import javax.swing.text.JTextComponent;

class BufferPool implements ActionListener
{
	JApplet applet;
  Container contentpane;
  FiFoAlgorithm fifo;
  LruAlgorithm Lru;
  LfuAlgorithm Lfu;
  
  JPanel bottomPanel;
  JComboBox AlgoChoice;
  JTextField Input;
  JButton Insert;
  JButton Next;
  JButton Main;
  
  JPanel textPanel;
  JTextPane textPane;
  String displayText[];
  int displayPointer;
  JScrollPane TextAreaScrollPane;
  
  drawBuffer bufferImage;
  
  Document doc;
  String [] Styles = { "Blue", "Red", "Orange", "red", "LightGray" };
  Style def;
  Style Blue;
  Style Red;
  Style Orange;
  Style red;
  Style LightGray;
  
  int buttonWidth;
  int buttonHeight;
  int screenwidth;
  int screenheight;
  
  int diskSpace [];
  int Access [];
  int MainMemory [];
  int MainPointer=0;
  int PageToDelete = -1;
  String AlgoSelected = null;
  boolean finishRunning;
  boolean ShowPageToDelete;
  
  int AnElement =-1; // THe value of the element that is entered..
  
  boolean LruShowPage = true;
  boolean LFUShowPage = true;
  
  public BufferPool ( BufferMain space )
  {
    applet = space;
    bottomPanel = new JPanel ( );
    contentpane = space.getContentPane();
    contentpane.setLayout (null);
    buttonWidth = 70;
    buttonHeight = 70;
    
    AlgoChoice = new JComboBox ( );
    AlgoChoice.setEditable ( false );
    AlgoChoice.setEnabled ( true );
    
    AlgoChoice.addItem ( "" );
    AlgoChoice.addItem ( "FIFO" );
    AlgoChoice.addItem ( "LRU" );
    AlgoChoice.addItem ( "LFU" );
    
    AlgoChoice.addActionListener(new ActionListener() // for the Combobox.
    {
      public void actionPerformed(ActionEvent e)
      {
        JComboBox Algo = (JComboBox)e.getSource();
        AlgoSelected = (String)Algo.getSelectedItem();
        if(AlgoSelected.equals("")) {
          return;
        }
        Input.setEnabled ( true );
        Input.setEditable ( true );  
        Main.setEnabled ( true );  
        Insert.setEnabled ( true ); 
        bufferImage.SetAlgoSelected ( AlgoSelected );
        bufferImage.repaint ( );
        displayPointer = 0;
        textPane.setText ( " " );
        
        displayText [displayPointer++] = "\n Algorithm Selected is " + AlgoSelected + "\n";
        Message ( "Red" );         
      }
    });
    
    Insert = new JButton ( "Insert" );
    Insert.setActionCommand ( "Insert" );
    
    Main = new JButton ( "Restart" );
    Main.setActionCommand ( "Main" );
    
    Next = new JButton ( "Next" );
    Next.setActionCommand ( "Next" );
    
    Input = new JTextField ( 5 );
    Input.setActionCommand ( "Input" );
    
    Insert.setEnabled ( false );
    Main.setEnabled ( false );
    Input.setEditable ( false );
    Input.setEnabled ( false );
    Next.setEnabled ( false );
    
    Font F = new Font ("Arial", Font.PLAIN, 17);
    Font newfont = new Font ("Arial", Font.PLAIN, 9);
    
    Input.setFont ( F );
    Insert.setFont ( newfont );
    AlgoChoice.setFont ( newfont );
    Main.setFont ( newfont );
    Next.setFont ( newfont );
    
    AlgoChoice.setToolTipText ( " Allows you to select an Algorithm " );
    Input.setToolTipText ( " Enter a page number from 0-9" );
    Insert.setToolTipText ( " Inserts page into Main Memory " );
    Next.setToolTipText ( " Moves to next step in the Algorithm " );
    Main.setToolTipText ( " Allows you to Restart " );
    
    
    Input.addActionListener ( this );
    Insert.addActionListener ( this );
    Main.addActionListener ( this );
    Next.addActionListener ( this );
    
    // Display the screen based on the applet width and height.
    
    screenheight = applet.getHeight();
    screenwidth = applet.getWidth();

    double posx  = 3.0/5.0*screenwidth;
    double posy = 7.0/8.0 * screenheight;
    double sizex = 2.0/5.0*screenwidth;
		
    int positionX = (int) posx;
    int positionY = (int) posy;
    int sizeX = (int) sizex;
    
    bottomPanel.setSize ( screenwidth-10, 40 );
    bottomPanel.setLocation( 1,screenheight*7/8);
    
    bottomPanel.add ( AlgoChoice );
    bottomPanel.add ( Input );
    bottomPanel.add ( Insert );
    bottomPanel.add ( Next );
    bottomPanel.add ( Main );
    
    textPane = new JTextPane ( );
    textPane.setEditable ( false );
    textPane.setRequestFocusEnabled ( true );
    textPane.setText ( " " );
    
    double XX =  screenwidth*2.0/5.00;
    double YY =  screenheight*7.0/8.0;
    int xX = ( int ) XX;
    int yY = ( int ) YY;
    
    TextAreaScrollPane = new JScrollPane ( textPane );
    TextAreaScrollPane.setVerticalScrollBarPolicy(
                        JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
    TextAreaScrollPane.setPreferredSize ( new Dimension ( sizeX-5, positionY-5) );
    
    textPanel = new JPanel();
    textPanel.add(TextAreaScrollPane);
    textPanel.setLocation(positionX+2, 1);
    textPanel.setSize( xX+ 10,yY+2 );//Panel for ScrollPane.
    
    displayText = new String [1000];
    displayText [displayPointer ] = "Select an Algorithm to run";
    displayPointer ++;
    
    doc = textPane.getDocument ();

	def = StyleContext.getDefaultStyleContext().
                                        getStyle(StyleContext.DEFAULT_STYLE);
    Blue = textPane.addStyle ( "Blue",def );
    StyleConstants.setForeground ( Blue, Color.blue );
    StyleConstants.setFontFamily ( Blue, "Arial" );
    StyleConstants.setFontSize ( Blue, 14 );
    
    Red = textPane.addStyle ( "Red", def );
    StyleConstants.setForeground ( Red, Color.red );
    StyleConstants.setAlignment( Red, StyleConstants.ALIGN_CENTER); 
    StyleConstants.setFontFamily ( Red, "Arial" );
    StyleConstants.setFontSize ( Red, 14 );
    StyleConstants.setBold ( Red, true );
    
    Orange = textPane.addStyle ( "Orange", def );
    StyleConstants.setForeground ( Orange, Color.orange );
    StyleConstants.setFontFamily ( Orange, "Arial" );
    StyleConstants.setFontSize ( Orange, 14 ); 
    
    red = textPane.addStyle ( "red", def );
    StyleConstants.setForeground ( red, Color.red );
    StyleConstants.setFontFamily ( red, "Arial" );
    StyleConstants.setFontSize ( red, 14 );
    StyleConstants.setBold ( red, false ); 
   
    Message ( "Red" );
    
    diskSpace = new int [10];
    MainMemory = new int[5];
    Access = new int[10];
    
    for ( int i=0; i<10; i++ )
      diskSpace[i] = -1;
    
    for ( int i=0; i<5; i++ )
      MainMemory[i] = -1;
    
    bufferImage = new drawBuffer ( screenwidth*3/5 , screenheight*7/8, diskSpace, MainMemory, Access );
    bufferImage.setLocation ( 0,2 );

    contentpane.add ( bottomPanel );
    contentpane.add ( textPanel );
    contentpane.add ( bufferImage );
    
  }
  
  public void actionPerformed ( ActionEvent ae )
  {
  	String message = null;
    message = ae.getActionCommand();    
    boolean invalid = false;   
    String text = null;
        
    if ( message.equals ( "Input" ) );
    {
      try
      {
        text = Input.getText ().trim ();
        Insert.setEnabled ( true );
        AnElement = Integer.parseInt ( text );
        AlgoChoice.setEnabled ( false );

        if ( AnElement == 0 )
        {
          if ( ! ( text.equals ( "0" ) ) )
            invalid = true;
        }
          
      }
      catch (NumberFormatException e )
      {
      	//displayText[displayPointer++] =  "\n\n Invalid Input. Number Format Exception Occured";
        //Message ( "Red" );     	
      	Main.setEnabled ( true );
      }
    }
    
    if ( message.equals ( "Insert" ) )
    {
      if ( AnElement == -1 )
      {
      	displayText [displayPointer++ ] = "\n Enter a number first \n ";
      	Message ( "Red" );
      	return;
      }

      if ( ( AnElement > 9 ) || ( AnElement < 0 ) )
      {
        displayText[displayPointer ++] = "\n Invalid Input " + "\n Enter numbers between 0-9.";
        Message ( "Red" );        
        Input.setText ( " " );
      }
      else
      { 
        displayText[displayPointer++] =  "\n\n The Element entered is " + displayText[displayPointer].valueOf ( AnElement );
        Message ( "Blue" );
        bufferImage.repaint ( );
        bufferImage.PageInserted ( AnElement, "Page To Insert" );  
        Input.setText ( " " );
        Input.setEnabled ( false );
        Input.setEditable ( false );
        Insert.setEnabled ( false );
        Next.setEnabled ( true );
        finishRunning = false;
        ShowPageToDelete = true;
        LruShowPage = true;
        LFUShowPage = true;
      }
    }

    if ( message.equals ( "Next" ) )
    {
      if ( finishRunning == true )
      {
      	 Next.setEnabled ( false );
      	 Insert.setEnabled ( true );
      	 Input.setEditable ( true );
      	 Input.setEnabled ( true );
      	 
      	 bufferImage.ResetPageInserted ( );
         bufferImage.ResetInsertMark ( );
      	 displayText [displayPointer++] = "\n\n Page Insertion is complete.\n Enter next page ";
      	 Message ( "Red" );
      	 AnElement = -1;
      }
      else
      {
      	Insert.setEnabled ( false );
        if ( AlgoSelected.equals ( "FIFO" ) )
        {
          if ( fifo == null )
          {
            fifo = new FiFoAlgorithm ( diskSpace, MainMemory );
          }
      	
    	  FiFoImplementation ( );
        }
        
        else if ( AlgoSelected.equals ( "LRU" ) )
        {
          if ( Lru == null )
          {
          	Lru = new LruAlgorithm ( diskSpace, MainMemory );
          	LruShowPage = true;
          }
          
          LruImplementation ( );        
        }
        
        else if ( AlgoSelected.equals ( "LFU" ) )
        {
          if ( Lfu == null )
          {
            Lfu = new LfuAlgorithm ( diskSpace, MainMemory, Access );
          }
       
          LfuImplementation ( );
        }
          
      }
    }
      
    if ( message.equals ( "Main" ) )
    {
      AlgoChoice.setSelectedIndex(0);
      diskSpace = null;
      MainMemory = null;
      Access = null;
      diskSpace = new int[10];
      MainMemory = new int[5];
      Access = new int [10];
      MainPointer = 0;
      Input.setText ( " " );
      fifo = null;
      Lru = null;
      Lfu = null;
      
      displayText = null;
      displayText = new String [1000];
      displayPointer=0;
      textPane.setText ( " " );
      
      for ( int i=0; i<5; i++ )
        MainMemory[i] = -1;
      for ( int i=0; i<10; i++ )
        diskSpace [i] = -1;
        
      contentpane.remove ( bufferImage );
      contentpane.repaint ( );
      bufferImage = null;
      bufferImage = new drawBuffer ( screenwidth*3/5 , screenheight*7/8, diskSpace, MainMemory, Access );
      contentpane.add ( bufferImage );
      contentpane.repaint ( );
      AlgoChoice.setEnabled ( true );
      Input.setEnabled ( false );
      Input.setEditable ( false );
      Insert.setEnabled ( false );
      Next.setEnabled ( false );
      Main.setEnabled ( false );
      applet.getContentPane().repaint ( );
      applet.getContentPane().validate ( );
      
      displayText [displayPointer++ ] = "Select an Algorithm to run";
      Message ( "Red" );
    }
  }
  
  public void FiFoImplementation ( )
  {
    boolean IsElementthere = false;
    int PageNumber =0;
    
    for ( int i=0; i<5;i++ )
    {
     if ( MainMemory[i] == AnElement )
     {
       IsElementthere= true;
       PageNumber = i;
     }
    }
    
    if ( IsElementthere == true )
    {
      displayText[displayPointer++] = "\n Page alread in Memory";
      Message ( "Red" );
      bufferImage.InsertPage ( PageNumber );
      finishRunning = true;
    }
    else
    {   
      if ( MainPointer < 5 )
      {      	
        fifo.InsertElement ( AnElement, MainPointer );
        bufferImage.InsertPage ( 0 );
        MainPointer++;
        displayText[displayPointer++] = "\n Page inserted in memory is " + displayText[displayPointer].valueOf ( AnElement );
        Message ( "Blue" );
        displayText [displayPointer++] = "\n Space in main memory. So no need to remove any page";
        finishRunning = true;
      }
      else
      {
      	if ( ShowPageToDelete == true )
      	{ 
          bufferImage.DeleteMarked ( 4 );
          ShowPageToDelete = false;
        }
      	
      	else
      	{       	 
          displayText[displayPointer++] = "\n Page removed from memory is " + displayText[displayPointer].valueOf ( MainMemory[4] );
          bufferImage.PageInserted ( MainMemory[4], "Page Deleted" );
          Message ( "Blue" );
          displayText[displayPointer++] = "\n Page inserted in memory is " + displayText[displayPointer].valueOf ( AnElement );
          Message ( "Blue" );        
          fifo.InsertElement ( AnElement, MainPointer );
          bufferImage.ResetDeleteMarked ( );
      	  bufferImage.InsertPage ( 0 );
      	  finishRunning = true;
        }
      }        
    }    
  }
  
  public void LruImplementation ( )
  { 
    
    if ( AnElement == MainMemory [0] )
    {
      bufferImage.InsertPage ( 0 );
      displayText[displayPointer++] = "\n\n Page is already in its place.\n";
      Message ( "Red" );      
      finishRunning = true;
      return;
    }

  	if ( MainPointer < 5 )
  	{
  	  if ( Lru.IsElementThere ( AnElement, MainPointer ) )
  	  {
  	  	if ( LruShowPage == true )
  	  	{
  	  	  int index = Lru.Index ( AnElement, MainPointer );
  	  	  bufferImage.InsertPage ( index );
  	  	  LruShowPage = false;
  	  	  return;
  	    }
  	    
  	    bufferImage.ResetInsertMark ( );
  	  	Lru.SetIsElement ( false );
  	  	Lru.InsertElement ( AnElement, MainPointer );
  	  	bufferImage.InsertPage ( 0 );
  	  	displayText[displayPointer++] = "\n Page is already in memory and has been brought in front.";
        Message ( "Red" );	 
        finishRunning = true; 	
  	  }
  	  
  	  else
  	  {
  	    displayText[displayPointer++] = "\n Page inserted in memory is " + displayText[displayPointer].valueOf ( AnElement );
        Message ( "Blue" );
        Lru.SetIsElement ( false );
        Lru.InsertElement ( AnElement, MainPointer );
        MainPointer++;
        bufferImage.InsertPage ( 0 );
        finishRunning = true;
      } 	  
    }
    else
    {
      if ( Lru.IsElementThere ( AnElement, MainPointer) )
      {
      	if ( LruShowPage == true )
      	{
      	  int index = Lru.Index ( AnElement, MainPointer );
  	  	  bufferImage.InsertPage ( index );
  	  	  LruShowPage = false;
  	  	  return;
  	    }

   	    bufferImage.ResetInsertMark ( );
  	  	Lru.SetIsElement ( false );
  	  	Lru.InsertElement ( AnElement, MainPointer );
  	  	bufferImage.InsertPage ( 0 );

        displayText[displayPointer++] = "\n\n Page is Already in the memory has been brought in front. " + displayText[displayPointer].valueOf ( AnElement );
        Message ( "Red" ); 
        finishRunning = true;      
      }
      else
      {
      	if ( LruShowPage == true )
      	{
      	  bufferImage.DeleteMarked ( 4 );
      	  LruShowPage = false;
      	  return;
        }
        
        bufferImage.ResetDeleteMarked ( );
      	bufferImage.PageInserted ( MainMemory [4], "Page Deleted" );
      	
      	displayText[displayPointer++] = "\n Page removed is " + displayText[displayPointer].valueOf ( Lru.Pageremoved ( ) ) ;
        Message ( "Blue" );
        displayText[displayPointer++] = "\n Page added is " + displayText[displayPointer].valueOf( AnElement );
        Message ( "Blue" ); 
        
      	Lru.SetIsElement ( false );
        Lru.InsertElement ( AnElement, MainPointer );     
        
        bufferImage.InsertPage ( 0 );
        finishRunning = true;  
      }
    }
  }
  
  public void LfuImplementation ( )
  {
  	if ( MainPointer < 5 )
  	{
  	  if ( Lfu.IsElementInMemory ( AnElement, MainPointer ) )
  	  {
  	  	 if ( LFUShowPage == true ) 
  	  	 {
  	  	   int index = Lfu.Index ( AnElement, MainPointer );
  	  	   bufferImage.InsertPage ( index );
  	  	   LFUShowPage = false;
  	  	   bufferImage.repaint ( );
  	  	   return;
  	  	 }
  	  	 
  	  	 else 
  	  	 {
  	  	   Lfu.ResetIsInMemory ( false );
           bufferImage.ResetInsertMark ( );
  	  	   Lfu.InputElement ( AnElement, MainPointer ); 
  	  	   int index = Lfu.Index ( AnElement, MainPointer );
  	  	   bufferImage.InsertPage ( index );
  	  	   finishRunning = true;
  	  	   return;
  	  	 }
  	  }
  	  
  	  else
  	  {
  	  	 Lfu.ResetIsInMemory ( false );
  	  	 Lfu.InputElement ( AnElement, MainPointer );
  	  	 int index = Lfu.Index ( AnElement, MainPointer );
  	  	 bufferImage.InsertPage ( index );
  	  	 finishRunning = true;
  	  	 MainPointer++;
  	  }
    }
    else
    {
      if ( Lfu.IsElementInMemory ( AnElement, MainPointer ) )
      {
      	if ( LFUShowPage == true )
      	{
      	   int index = Lfu.Index ( AnElement, MainPointer );
  	  	   bufferImage.InsertPage ( index );
  	  	   LFUShowPage = false;
  	  	   bufferImage.repaint ( );
  	  	   return;
  	  	}
  	  	
  	  	else 
  	  	{
  	  	   Lfu.ResetIsInMemory ( false );
  	  	   Lfu.InputElement ( AnElement, MainPointer ); 
           bufferImage.ResetInsertMark ( );
  	  	   int index = Lfu.Index ( AnElement, MainPointer );
  	  	   bufferImage.InsertPage ( index );
  	  	   finishRunning = true;
  	  	   return;
  	  	 }
  	  }

      else
      {
        if ( LFUShowPage == true )
        {
          bufferImage.DeleteMarked ( 4 );
      	  LFUShowPage = false;
      	  return;
        }
      
        else
        {
      	  bufferImage.ResetDeleteMarked ( );
      	  bufferImage.PageInserted ( MainMemory [4], "Page Deleted" );
      	
      	  displayText[displayPointer++] = "\n Page removed is " + displayText[displayPointer].valueOf ( MainMemory [4] ) ;
          Message ( "Blue" );
          displayText[displayPointer++] = "\n Page added is " + displayText[displayPointer].valueOf( AnElement );
          Message ( "Blue" ); 
        
          Lfu.ResetIsInMemory ( false );
          Lfu.InputElement ( AnElement, MainPointer );
          int index = Lfu.Index ( AnElement, MainPointer );
          bufferImage.InsertPage ( index );
          bufferImage.repaint ( );
          finishRunning = true;
          return;
        }
      }
    }
  }
  
  public void Message ( String color )
  {
  	try
    {
      doc.insertString(doc.getLength(), displayText[displayPointer-1],textPane.getStyle ( color ));
    }
    
    catch (BadLocationException ble)
    {
      System.out.println("Couldn't insert initial text.");
    }
  }  	
  	
} 


public class BufferMain extends JApplet
{
  public void init ( )
  {
    BufferPool set = new BufferPool ( this );
  }	
} 
    
    
    
    
    
  	
  