/*
Part of the algoviz@vt collection of algorithm visualizations.
Copyright (C) 2007  Virginia Tech Department of Computer Science

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.awt.Graphics2D.*;
import java.awt.image.BufferedImage;
import javax.swing.BorderFactory;
import java.lang.Object;

public class drawBuffer extends JComponent
{
  BufferedImage IMG;
  int totalHeight; // The total height of the entire display
  int totalWidth;  // The total width of the entire display
  
  int diskSpace[];
  int Access [];
  int diskWidth;
  int diskHeight;
  int diskX;
  int diskY;
  
  int MainMemory[];
  int MainWidth;
  int MainHeight;
  int MainX;
  int MainY;
  int AnElement;
  boolean deleteMark = false;
  int deleteNumber = -1;
  String AlgoSelected = "Null";
  boolean pageToInsert = false;
  boolean InsertMark = false;
  String InsertOrDelete = null;
  int InsertNumber;
    
  public drawBuffer ( int width, int height, int[] a, int[] b, int [] c )
  {
  	totalHeight = height;
  	totalWidth = width;
  	
  	int X = totalWidth/10;
  	int Y = totalHeight/14;
  	
  	diskX = X;
  	diskY = Y;
  	diskWidth = 3*X;
  	diskHeight = 10*Y;
  	
  	MainX = 6*X;
  	MainY = Y;
  	MainWidth = 3*X;
  	MainHeight = 5*Y;
  	this.setSize ( totalHeight - 2, totalWidth - 2 );
  	
  	diskSpace = a;
  	MainMemory =b;
  	Access = c;
  }
  
  public void paint ( Graphics g )
  {
    IMG = new BufferedImage ( totalWidth-3,totalHeight-3,BufferedImage.TYPE_INT_RGB );
    Graphics2D G = IMG.createGraphics ();
    G.fillRect ( 0,0,totalWidth,totalHeight ); // This makes the background of the image of the color selected
    
    G.setColor ( Color.cyan );
    G.fillRect ( diskX, diskY, diskWidth, diskHeight );
    G.fillRect ( MainX, MainY, MainWidth, MainHeight );  
    
    if ( deleteMark == true )
    {
   	  int startY = MainY + deleteNumber*diskY;
   	  
   	  G.setColor ( Color.pink );
   	  G.fillRect ( MainX,startY, MainWidth, diskY );
   	}  
    
    if ( InsertMark == true )
    {
   	  int startY = MainY + InsertNumber*diskY;
   	  
   	  G.setColor ( Color.green );
   	  G.fillRect ( MainX,startY, MainWidth, diskY );
   	} 
    
    Font font = new Font ( "Arial", Font.BOLD, 16);
    G.setFont ( font );
    G.setColor ( Color.blue );

    String str = " ";
    for ( int i=0;i<10;i++ )
    {
      int Y = diskY + (i+1)*diskY;
      int X = diskX+3*diskX;
      
      str = str.valueOf ( i );
      int x = diskX + diskWidth/2;
      int y = Y - diskY/2;
      G.drawString ( str,x,y );
      if ( i!= 9 )
        G.drawLine ( diskX,Y,X,Y );
    }
   
    // draws the Main memory
    
    for ( int i=0;i<5;i++ )
    {
      int Y = MainY + ( i+1 )*MainY;
      int X = MainX + MainWidth;
      
      if ( MainMemory[i] != -1 )
      {
      	str = str.valueOf ( MainMemory[i] );
        int x = MainX + MainWidth/2;
        int y = Y - MainY/2;
        G.drawString ( str,x,y );
        int DX = diskX+ diskWidth;
        int DY = diskY + ( ( MainMemory[i]+1 ) * diskY ) - diskY/2;
        int Xx = MainX;
        G.drawLine ( Xx,y, DX, DY ); 
      }
      
      if ( i!= 4 )
        G.drawLine ( MainX,Y,X,Y );
    }

    int YY = diskHeight + 2*diskY +5;
    
    G.drawString ( "Physical Memory", diskX, YY );
    G.drawString ( "Main Memory", MainX+2, YY );
    if ( ! ( AlgoSelected.equals ( "Null" ) ) )
      G.drawString ( AlgoSelected, 15,21 );
    
    if ( AlgoSelected.equals ( "LFU :" ) )
    {
      G.drawLine ( 3*diskX,diskY,3*diskX,( diskHeight+diskY ));
      G.drawLine ( ( MainX + 2*diskX ), MainY, ( MainX + 2*diskX ), ( MainHeight + MainY) );
      
      for ( int i=0; i<10; i++ )
      {
      	str = str.valueOf ( Access [i] );
      	int x = 3*diskX + diskX/2;
      	int y = diskY + (i+1)*diskY - diskY/2;
        G.drawString ( str, x,y ); // Prints the number of access in the diskSpace
      }
      
      for ( int i=0; i<5; i++ )
      {
      	if ( MainMemory[i] == -1 )
      	  str = " ";
      	else
      	  str = str.valueOf ( Access [ MainMemory[i] ] );
      	        	  
      	int x = MainX + 2*diskX + diskX/2;
        int y= diskY + ( ( i+1 ) * diskY ) - diskY/2;
        G.drawString ( str,x,y ); // Prints the number of access in the Mainmemory..
      }
    } 
    
    if ( pageToInsert == true )
    {
      G.setColor ( Color.cyan );
      
      if ( InsertOrDelete.equals ( "Page Deleted :") )
        G.setColor ( Color.pink );
      else
        G.setColor ( Color.green );
      G.fillRect (  MainX, MainHeight+3*diskY, MainWidth, diskY );
      str = str.valueOf ( AnElement );
      G.setColor ( Color.blue );
      G.drawString ( str,MainX+MainWidth/2, MainHeight+3*diskY+diskY/2 );
      G.drawString ( InsertOrDelete , MainX, MainHeight+2*diskY+5 );
    }
    g.drawImage ( IMG,0,0,totalWidth,totalHeight,this );
    
  }
  
  public void SetAlgoSelected ( String str )
  {
  	AlgoSelected = str;
  	AlgoSelected = AlgoSelected + " :";
  	repaint ( );
  }
  
  public void PageInserted ( int AElement, String value )
  {
  	AnElement = AElement;
  	pageToInsert = true;
  	InsertOrDelete = value;
  	InsertOrDelete = value + " :";
  	repaint ( );
  }
  
  public void ResetPageInserted ( )
  {
  	pageToInsert = false;
  	repaint ( );
  }
  
  public void DeleteMarked ( int number )
  {
  	deleteMark = true;
  	deleteNumber = number;
  	repaint ( );
  }
  
  public void ResetDeleteMarked ( )
  {
  	deleteMark = false;
  	repaint ( );
  }
  
  public void InsertPage ( int number )
  {
  	InsertMark = true;
  	InsertNumber = number;
  	repaint ( );
  }
  
  public void ResetInsertMark ( )
  {
  	InsertMark = false;
  	repaint ( );
  }
  	
}

    
  
  
  
    
